from deepinfra.models.base import BaseModel
from deepinfra.types.text_generation.request import TextGenerationRequest
from deepinfra.types.text_generation.response import TextGenerationResponse


class TextGenerationBaseModel(BaseModel):
	def __init__(self, endpoint: str, auth_token: str):
		super().__init__(endpoint, auth_token)
		self.endpoint = endpoint
		self.auth_token = auth_token

	def generate(self, body: TextGenerationRequest) -> TextGenerationResponse:
		try:
			response = self.client.post(body)
			return response
		except Exception as error:
			print('Error generating text:', error)
			raise error


from dataclasses import dataclass
from typing import List, Optional, Dict


@dataclass
class InferenceStatus:
	status: str
	runtime_ms: int
	cost: float
	tokens_generated: int


@dataclass
class GeneratedText:
	generated_text: str


@dataclass
class TextGenerationResponse:
	request_id: str
	inference_status: InferenceStatus
	results: List[GeneratedText]
	num_tokens: int
	num_input_tokens: int





from dataclasses import dataclass, field
from typing import Optional, List, Dict

@dataclass
class TextGenerationRequest:
    input: str
    stream: Optional[bool] = None
    max_new_tokens: Optional[int] = None
    temperature: Optional[float] = None
    top_p: Optional[float] = None
    top_k: Optional[int] = None
    repetition_penalty: Optional[float] = None
    stop: Optional[List[str]] = None
    num_responses: Optional[int] = None
    response_format: Optional[Dict[str, str]] = None
    presence_penalty: Optional[float] = None
    frequency_penalty: Optional[float] = None
    webhook: Optional[str] = None


from dataclasses import dataclass
from typing import Optional, List

@dataclass
class EmbeddingsRequest:
	inputs: List[str]
	normalize: Optional[bool] = None
	image: Optional[str] = None
	webhook: Optional[str] = None


from deepinfra.models.base.text_generation import TextGenerationBaseModel


class Dolphin(TextGenerationBaseModel):
	"""
	Wrapper for the dolphin-2.6-mixtral-8x7b model.
	"""
	endpoint='https://api.deepinfra.com/v1/inference/cognitivecomputations/dolphin-2.6-mixtral-8x7b'

	def __init__(self, auth_token: str):
		super().__init__(Dolphin.endpoint, auth_token)




from deepinfra.models.base import ImageGenerationBaseModel


class Sdxl(ImageGenerationBaseModel):
	endpoint = 'https://api.deepinfra.com/v1/inference/stability-ai/sdxl'

	def __init__(self, auth_token: str):
		super().__init__(Sdxl.endpoint, auth_token)



from deepinfra.models.base.text_embedding import TextEmbeddingBaseModel


class GteLarge(TextEmbeddingBaseModel):
	endpoint = 'https://api.deepinfra.com/v1/inference/thenlper/gte-large'

	def __init__(self, auth_token: str):
		super().__init__(GteLarge.endpoint, auth_token)


from deepinfra.models.base.text_embedding import TextEmbeddingBaseModel


class BgeBaseEnV15(TextEmbeddingBaseModel):
	endpoint = 'https://api.deepinfra.com/v1/inference/BAAI/bge-base-en-v1.5'

	def __init__(self, auth_token: str):
		super().__init__(BgeBaseEnV15.endpoint, auth_token)



from .bge_base_en_v15 import BgeBaseEnV15
from .bge_large_en_v15 import BgeLargeEnV15
from .gte_large import GteLarge
from .gte_base import GteBase

__all__ = [
	"BgeBaseEnV15",
	"BgeLargeEnV15",
	"GteLarge",
	"GteBase",
]


from dataclasses import dataclass
from typing import Optional, List, Dict



@dataclass
class InferenceStatus:
    status: str
    runtime_ms: int
    cost: float
    tokens_generated: Optional[int] = None
    tokens_input: Optional[int] = None

@dataclass
class Metrics:
    predict_time: int

@dataclass
class ImageGenerationResponse:
    request_id: str
    inference_status: InferenceStatus
    input: Dict
    output: List[str]
    id: str
    started_at: str
    completed_at: str
    logs: str
    status: str
    metrics: Metrics
    webhook_events_filter: List[str]
    output_file_prefix: str
    version: Optional[str] = None
    created_at: Optional[str] = None
    error: Optional[str] = None
    webhook: Optional[str] = None



from dataclasses import dataclass
from typing import Optional, List


@dataclass
class EmbeddingStatus:
	status: str
	runtime_ms: int
	cost: float
	tokens_generated: int
	tokens_input: int

@dataclass
class EmbeddingsResponse:
	embeddings: List[List[float]]
	input_tokens: int
	inference_status: EmbeddingStatus
	request_id: Optional[str] = None



from deepinfra.models.base.text_generation import TextGenerationBaseModel


class Mixtral(TextGenerationBaseModel):
	"""
	Wrapper for the Mixtral-8x7B-Instruct-v0.1 model.
	"""
	endpoint='https://api.deepinfra.com/v1/inference/mistralai/Mixtral-8x7B-Instruct-v0.1'

	def __init__(self, auth_token: str):
		super().__init__(Mixtral.endpoint, auth_token)


from dataclasses import dataclass
from typing import Optional

@dataclass
class ImageGenerationRequest:
	prompt: str
	negative_prompt: Optional[str] = None
	image: Optional[str] = None
	mask: Optional[str] = None
	width: Optional[int] = None
	height: Optional[int] = None
	num_outputs: Optional[int] = None
	scheduler: Optional[str] = None
	num_inference_steps: Optional[int] = None
	guidance_scale: Optional[float] = None
	prompt_strength: Optional[float] = None
	seed: Optional[int] = None
	refine: Optional[str] = None
	high_noise_frac: Optional[float] = None
	refine_steps: Optional[int] = None
	apply_watermark: Optional[bool] = None


from deepinfra.clients import DeepInfraClient


class BaseModel:
    def __init__(self, endpoint, auth_token):
        self.endpoint = endpoint
        self.auth_token = auth_token
        self.client = DeepInfraClient(self.endpoint, self.auth_token)


from deepinfra.constants import MAX_RETRIES, USER_AGENT, INITIAL_BACKOFF, SUBSEQUENT_BACKOFF
import time
import requests

class DeepInfraClient:
	max_retries = MAX_RETRIES
	initial_backoff = INITIAL_BACKOFF
	subsequent_backoff = SUBSEQUENT_BACKOFF


	def __init__(self, url, auth_token):
		self.url = url
		self.auth_token = auth_token


	def backoff_delay(self, attempt):
		delay = self.initial_backoff if attempt == 1 else self.subsequent_backoff
		time.sleep(delay)


	def post(self, data, headers=None):
		if headers is None:
			headers = {}
		headers.update(
			{'Content-Type': 'application/json', 'Authorization': f'Bearer {self.auth_token}', 'User-Agent': USER_AGENT})

		for attempt in range(self.max_retries + 1):
			try:
				response = requests.post(self.url, json=data, headers=headers)
				response.raise_for_status()  # Will raise exception for 4xx/5xx errors
				return response
			except requests.RequestException as error:
				if attempt < self.max_retries:
					print(f'Request failed, retrying... Attempt {attempt + 1}/{self.max_retries}')
					self.backoff_delay(attempt + 1)
				else:
					raise error  # Re-throw error on last attempt

		raise Exception('Maximum retries exceeded')



from setuptools import setup, find_packages

setup(
	name='deepinfra',
	version='0.1.0',
	author='Oguz Vuruskaner',
	author_email='oguzvuruskaner@gmaikl.com',
	description='Unofficial Python wrapper for the DeepInfra Inference API',
	long_description=open('README.md').read(),
	long_description_content_type='text/markdown',
	url='https://github.com/ovuruska/deepinfra-python',
	packages=find_packages(),
	install_requires=[
		'requests',
	],
	include_package_data=True,

	use_scm_version=True,
	setup_requires=['setuptools_scm'],
	classifiers=[
		'Programming Language :: Python :: 3',
		'License :: OSI Approved :: MIT License',
		'Operating System :: OS Independent',
	],
)
